'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
const assert = require('assert');

//
class GoodsJoinActService extends CrudService {
  constructor(ctx) {
    super(ctx, 'goodsjoinact');
    this.model = this.ctx.model.Shop.GoodsJoinAct;
  }
  async query(filter, { skip = 0, limit, sort, desc, projection } = {}) {
    // 处理排序
    if (sort && _.isString(sort)) {
      sort = { [sort]: desc ? -1 : 1 };
    } else if (sort && _.isArray(sort)) {
      sort = sort.map(f => ({ [f]: desc ? -1 : 1 })).reduce((p, c) => ({ ...p, ...c }), {});
    }
    let condition = _.cloneDeep(filter);
    condition = await this.beforeQuery(condition);
    condition = this.dealFilter(condition);
    const { shop, status, platformAct, goods, goods_name, shop_name } = condition;
    const fMatch = { };
    if (shop)fMatch.shop = shop;
    if (platformAct) fMatch.platformAct = platformAct;
    if (goods)fMatch.goods = goods;
    if (status) fMatch.status = status;
    const pipeline = [];
    if (Object.keys(fMatch).length > 0) {
      pipeline.push({ $match: fMatch });
    }
    // 表关联
    pipeline.push({ $addFields: { shop_id: { $toObjectId: '$shop' }, goods_id: { $toObjectId: '$goods' } } });
    pipeline.push({
      $lookup: {
        from: 'shop',
        localField: 'shop_id',
        foreignField: '_id',
        pipeline: [{ $project: { name: 1 } }],
        as: 'shopInfo',
      },
    });
    pipeline.push({
      $lookup: {
        from: 'goods',
        localField: 'goods_id',
        foreignField: '_id',
        pipeline: [{ $project: { name: 1, file: 1 } }],
        as: 'goodsInfo',
      },
    });
    pipeline.push({ $unwind: '$shopInfo' });
    pipeline.push({ $unwind: '$goodsInfo' });
    // 过滤
    if (goods_name) {
      pipeline.push({ $match: { 'goodsInfo.name': new RegExp(goods_name) } });
    }
    if (shop_name) {
      pipeline.push({ $match: { 'shopInfo.name': new RegExp(shop_name) } });
    }
    pipeline.push({ $sort: { 'meta.createdAt': -1 } });
    // 找价格最低的规格的金额
    pipeline.push({
      $lookup: {
        from: 'goodsSpec',
        localField: 'goods',
        foreignField: 'goods',
        pipeline: [
          {
            $group: {
              _id: '$goods',
              data: { $min: '$$CURRENT' },
            },
          },
          { $project: { name: '$data.name', file: '$data.file', sell_money: '$data.sell_money', flow_money: '$data.flow_money' } },
        ],
        as: 'specInfo',
      },
    });
    pipeline.push({ $unwind: '$specInfo' });
    pipeline.push({
      $project: {
        platformAct: 1,
        shop: '$shopInfo',
        goods: '$goodsInfo',
        flow_money: { $toString: '$specInfo.flow_money' },
        sell_money: { $toString: '$specInfo.sell_money' },
        spec_file: '$specInfo.file',
      },
    });
    const qPipeline = _.cloneDeep(pipeline);
    if (parseInt(skip)) qPipeline.push({ $skip: parseInt(skip) });
    if (parseInt(limit)) qPipeline.push({ $limit: parseInt(limit) });
    const res = await this.model.aggregate(qPipeline);
    const tPipeline = _.cloneDeep(pipeline);
    tPipeline.push({ $count: 'total' });
    const total = await this.model.aggregate(tPipeline);
    // 整理图片
    // const list = [];
    // for (const r of res) {
    //   const i = _.cloneDeep(r);
    //   const { spec_file = [] } = i;
    //   const goods_file = _.get(r, 'goods.file', []);
    //   const nf = [ ...spec_file, ...goods_file ];
    //   delete i.spec_file;
    //   i.goods.file = nf;
    //   list.push(i);
    // }
    return { data: res, total: _.get(_.head(total), 'total', 0) };
  }
}

module.exports = GoodsJoinActService;
